from __future__ import with_statement
from path import path
from time import gmtime, strftime
import textwrap
import es
import cmdlib
import cfglib
import gamethread

info = es.AddonInfo()
info.name = 'Feedback'
info.version = 1.0
info.basename = 'feedback'
info.url = 'http://addons.eventscripts.com/addons/view/feedback'
info.author = 'Dead Man Walker'

CFG_DIR = path(es.ServerVar('eventscripts_gamedir')).joinpath('cfg').joinpath(info.basename)
CONFIG_PATH = CFG_DIR.joinpath(info.basename + "_config.cfg")
FEEDBACK_PATH = CFG_DIR.joinpath(info.basename + ".txt")
FEEDBACK_AMOUNT_DICT = {
 0 : 'No limit',
 1 : 'Once every spawn',
 2 : 'Once every round',
 3 : 'Once every connect/ mapchange'
}

if not path.exists(CFG_DIR):
   path.mkdir(CFG_DIR)

cfg = cfglib.AddonCFG(CONFIG_PATH)
cfg.text("*****************************************************************")
cfg.text("*** %s Config" %info.name)
cfg.text("*****************************************************************")
cfg.text(' ')
FEEDBACK_COMMANDS = cfg.cvar('feedback_commands', '!feedback;!f;!fb', 'Chat command(s) to open the feedback box. Seperate with a semicolor (;)')
FEEDBACK_AMOUNT = cfg.cvar('feedback_amount', 2, 'How often a player may send a feedback. See the list below')
cfg.text(' ')
cfg.text('List of available limits')
feedback_amount_list = FEEDBACK_AMOUNT_DICT.keys()
feedback_amount_list.sort()
for x in feedback_amount_list:
   cfg.text(' -%s (%s)' %(x, FEEDBACK_AMOUNT_DICT[x]))
cfg.write()
cfg.execute()

users = set()
just_sent = set()

COMMANDS = str(FEEDBACK_COMMANDS).strip().split(';')

def load():
   if not COMMANDS:
      return
   for cmd in COMMANDS:
      cmdlib.registerSayCommand(cmd, openInputbox, "opens the inputbox")
   cmdlib.registerClientCommand('!feedback', printFeedback, "prints the feedback to file")

def unload():
   if not COMMANDS:
      return
   for cmd in COMMANDS:
      cmdlib.unregisterSayCommand(cmd)
   cmdlib.unregisterClientCommand('!feedback')

def es_map_start(event_var):
   users.clear()

def round_end(event_var):
   if int(FEEDBACK_AMOUNT) == 2:
      users.clear()

def player_spawn(event_var):
   userid = int(event_var['userid'])
   if int(FEEDBACK_AMOUNT) == 1 and userid in users:
      users.discard(userid)

def printFeedback(userid, args):
      if not str(args) or userid in just_sent:
         return
      # <Bug fix since every input gets fired twice
      just_sent.add(userid)
      gamethread.delayed(0.2, just_sent.discard, userid)
      # >
      if int(FEEDBACK_AMOUNT) in (1,2,3):
         users.add(userid)
      wrapper = textwrap.fill(str(args))
      es.tell(userid, '#multi', '#green[Info]: #lightgreenYour feedback has been sent!')
      with FEEDBACK_PATH.open('a') as f:
         f.write('--------------------------------------------------------------\n')
         f.write('%s [%s] (%s): \n' %(es.getplayername(userid), es.getplayersteamid(userid), strftime("%A, %d.%m.%y, %H:%M", gmtime())))
         f.write('--------------------------------------------------------------\n')
         f.write(wrapper + '\n')
         f.write('\n')   
         f.write('\n')    
    

def openInputbox(userid, args):
   if userid in users:
      if int(FEEDBACK_AMOUNT) == 1:
         limit = 'spawn'
      elif int(FEEDBACK_AMOUNT) == 2:
         limit = 'round'
      else:
         limit = 'connect'
      es.tell(userid, '#multi', '#green[Feedback]: #lightgreenYou can only send #darkgreenone #lightgreen feedback every #darkgreen%s!' %limit)
   else:
      es.tell(userid, '#multi', '#green[Feedback]: #lightgreenHit "ESC" and type in your feedback')
      es.escinputbox(20, userid, "Feedback", "Tell us your feedback", "!feedback ")  

def player_disconnect(event_var):
   users.discard(event_var['userid'])
     
